library(xtable)

##Bootstrapping function.
bootStrap <- function(vect, n){
  res <- vector();
  res <- sapply(1:n, function(x,vect){mean(sample(vect,length(vect),replace=TRUE))}, vect)
  ##for (i in 1:n)
  ##  res[i] <- mean(sample(vect, length(vect), replace=TRUE))
  res<-sort(res, decreasing=FALSE)
  return(c(res[n*0.025], res[n*0.975]))
}

##Counts the number of bootstrap instances that violate the null hypothesis
bootStrapCounter <- function(vect1, vect2, n){
  ans <- c(0,0);
  for (i in 1:n){
    samp1 <- sample(vect1, length(vect1), replace=TRUE)
    samp2 <- sample(vect2, length(vect2), replace=TRUE)    
    if (mean(samp1)>mean(samp2))
      ans[1] <- ans[1]+1
    else
      ans[2] <- ans[2]+1;
  }
  return(ans)
}

##permutation test
permutationTest <- function(vect1, vect2, n){
  dif=mean(vect1)-mean(vect2)
  ans <- vector();
  for (i in 1:n){
    tv1=vect1;
    tv2=vect2;
    for (j in 1:length(vect1)){
      if (runif(1)>0.5){
        temp <- tv1[j]
        tv1[j] <- tv2[j]
        tv2[j] <- temp
      }
    }
    ans[i] <- mean(tv1)-mean(tv2);
  }
  return(length(which(ans>dif))/n)
}

##A different implementation trying to avoid for loops... not useful yet.
permutationTest2 <- function(vect1, vect2, n){
  tmpfunc <- function(ii,vect1, vect2, n){
    tv1=vect1;
    tv2=vect2;
    for (j in 1:length(vect1)){
      if (runif(1)>0.5){
        temp <- tv1[j]
        tv1[j] <- tv2[j]
        tv2[j] <- temp
      }
    }
    return(mean(tv1)-mean(tv2))
  }
  dif=mean(vect1)-mean(vect2)
  ans <- vector();
  ans=unlist(lapply(1:n, tmpfunc, vect1, vect2,n));
  return(length(which(ans>dif))/n)
}

##Algorithm names
an=c('BL*', 'CG*', 'T99', 'CONTRAFold1.1', 'DIM-CG', 'NOM-CG', 'BL-FR*', 'MaxExpect', 'CONTRAFold2.0', 'CentroidFold', 'AveRNA')
##Citations. 'nnnn' and 'iiii' are strings that should be replaced by "\cite{" and "}" in the latex document. This is hack to fix a bug in the xtable package.
citations <- c('nnnnandronescu2008computationaliiii','nnnnandronescu2008computationaliiii','nnnnmathews1999expandediiii','nnnndo2006contrafoldiiii','nnnnandronescu2008computationaliiii','nnnnandronescu2008computationaliiii','nnnnandronescu2008computationaliiii','nnnnlu2009improvediiii','nnnndo2006contrafoldiiii','nnnnhamada2009predictioniiii','-')

##Read the input
mat<-as.matrix(read.csv('mat.csv',header=FALSE))

##Order the algorithms by their F-measure
algo.inds <- sort(colMeans(mat), index.return=TRUE, decreasing=TRUE)$ix

##Calculate mean F-measures and CIs
res <- matrix(0, length(mat[1,]), 3);
CI <- matrix(0, length(mat[1,]), 2);
for (i in 1:length(mat[1,])){
  bs <- bootStrap(mat[,i],1000)
  res[i,1] <- sprintf('%.3f',mean(mat[,i]))
  res[i,2] <- sprintf('(%.3f, %.3f)',bs[1], bs[2])
  res[i,3] <- citations[i];
  CI[i,] <- bs;
}
res <- res[algo.inds,]
rownames(res) <- an[algo.inds];
colnames(res) <- c('Mean F-measure', 'Confidence Interval', 'Citation')
sink(file='../DisplayItems/Bootstrap.tex', append=FALSE, 'output', FALSE);
xtable(res, label='AveRNA:Bootstrap', caption='F-measure values for different algorithms. 95 percent confidence intervals, calculated using bootstrapping, are shown in paranthesis.');
sink(NULL);


##Count the number of times that the null hypothesis is violated in bootstrapping of algorithms 7 and 11. This is used in somewhere in the manuscript.
bootStrapCounter(mat[,7], mat[,11],10000)


##Generate the figure with overalpping CIs.
postscript('../DisplayItems/CIs.eps', horizontal=FALSE)
ind <- rev(algo.inds);
par(mar = c(5, 10, 4, 2) + 0.1) 
xlim=c(0.55,0.75);
ylim=c(0,11)
plot(CI[ind,1], 1:length(CI[,1]), xlim=xlim, ylim=ylim, pch=20, col='red', axes=FALSE, xlab='', ylab='', cex=1.5);
par(new=TRUE);
plot(CI[ind,2], 1:length(CI[,1]), xlim=xlim, ylim=ylim, pch=20, col='red', axes=FALSE, xlab='', ylab='', cex=1.5);
par(new=TRUE);
text(rowMeans(CI[ind,]), 1:length(CI[,1]), xlim=xlim, ylim=ylim, label='|', col='black', axes=FALSE, xlab='', ylab='', cex=1);
for (i in 1:length(CI[,1])){
  segments(CI[ind[i],1],i, CI[ind[i],2],i, axes=FALSE, lwd=2);
  par(new=TRUE);
}
axis(1);
axis(2, labels=FALSE, at=1:length(CI[,1]))
title(xlab='Confidence Interval')
text(par("usr")[1] - 0.01, srt = 0, 1:length(CI[,1]), adj = 1, labels = an[ind], xpd = TRUE)
rect(0.578, 0.586, 0.619, 2.365, lwd=2, border=2)
rect(0.639, 4.546, 0.676, 6.420, lwd=2, border=2)
dev.off();


##Generate the table of Spearman correlations.
res <- matrix(0,length(mat[1,]),length(mat[1,]));
for (i in 1:length(mat[1,])){
  for (j in 1:length(mat[1,])){
    res[which(algo.inds==i),which(algo.inds==j)]=cor.test(mat[,i], mat[,j], method="spearman")$estimate 
  }
}
res2 <- matrix(0,length(mat[1,]),length(mat[1,]));
for (i in 1:length(mat[1,]))
  for (j in 1:length(mat[1,]))
    res2[i,j] <- min(res[i,j], res[j,i])
res2 <- format(res2,digits=3);
for (i in 1:length(mat[1,]))
  for (j in 1:length(mat[1,]))
    if (i<=j)
      res2[i,j] <- ''
rownames(res2) <- an[algo.inds];
colnames(res2) <- an[algo.inds];
sink(file='../DisplayItems/SpearmanC.tex', append=FALSE, 'output', FALSE);
xtable(res2, label='AveRNA:SpearmanC', caption='Spearman correlations between pairs of algorithms');
sink(NULL);


##Generate the table of permutation tests.
mat<-as.matrix(read.csv('mat.csv',header=FALSE, sep=','))
res <- matrix(0,length(mat[1,]),length(mat[1,]));
for (i in 1:length(mat[1,])){
  for (j in 1:length(mat[1,])){
    ##print(c(i,j));
    if (i<=j)
      res[i,j] <- ''
    if (i>j)
##Permutation test is performed for only 10 iterations to save time. In the manuscript 10^4 iterations were used.
      res[which(algo.inds==i),which(algo.inds==j)]=permutationTest(mat[,i], mat[,j], 10^1) 
  }
}
res2 <- format(res,digits=3);
tres <- matrix(as.numeric(res),dim(res)[1], dim(res)[2]);
for (i in 1:length(mat[1,])){
  for (j in 1:length(mat[1,])){    
    if (i>j)
      tres[i,j]=min(c(tres[i,j], tres[j,i], 1-max(tres[i,j], tres[j,i], na.rm=TRUE)),na.rm=TRUE)
    if (i<=j)
      tres[i,j] <- NA
  }
}
tmpinds <- vector();
for (i in 1:length(mat[1,]))
  tmpinds[i] <- which(algo.inds==i)
colnames(res2) <- an[algo.inds];
rownames(res2) <- an[algo.inds];
sink(file='../DisplayItems/permtest.tex', append=FALSE, 'output', FALSE);
print(xtable(res2, label='permtest', caption='P-values for permutation tests over every pair of algorithms'));
sink(NULL);


##Generate the PPV-Sensitivity plot with limited axis range
postscript('../DisplayItems/ppvsen.eps', horizontal=FALSE)
xlim=c(0.39, 0.9);
ylim=c(0.39, 0.9);
mat <- as.matrix(read.csv('ppvsen.csv', header=FALSE,sep=';'))
for (i in 1:(length(mat[1,])/2)){
  ind <- (i-1)*2
  plot(mat[,c(ind+1,ind+2)], col=i, xlim=xlim, ylim=ylim, pch=i, axes=FALSE, xlab='', ylab='', lty=1, lwd=1, cex=1.5)
  par(new=TRUE)
  if (length(which(!is.na(mat[,ind+1])))>1){
    plot(mat[,c(ind+1,ind+2)], col=i, type='l', xlim=xlim, ylim=ylim, axes=FALSE, xlab='', ylab='', lty=1, lwd=2)
    par(new=TRUE)
  }
}
axis(1); axis(2);
title(xlab='Positive Predictive Value', ylab='Sensitivity')
legend(0.8,0.9,legend=an,pch=1:length(mat[1,]), col=1:length(mat[1,]),cex=1)
grid();
dev.off();

##Generate the PPV-Sensitivity plot with full axis range
postscript('../DisplayItems/Supppvsen.eps', horizontal=FALSE)
xlim=c(0, 1);
ylim=c(0, 1);
mat <- as.matrix(read.csv('ppvsen.csv', header=FALSE,sep=';'))
for (i in 1:(length(mat[1,])/2)){
  ind <- (i-1)*2
  plot(mat[,c(ind+1,ind+2)], col=i, xlim=xlim, ylim=ylim, pch=i, axes=FALSE, xlab='', ylab='', lty=1, lwd=1, cex=1.5)
  par(new=TRUE)
  if (length(which(!is.na(mat[,ind+1])))>1){
    plot(mat[,c(ind+1,ind+2)], col=i, type='l', xlim=xlim, ylim=ylim, axes=FALSE, xlab='', ylab='', lty=1, lwd=2)
    par(new=TRUE)
  }
}
axis(1); axis(2);
title(xlab='Positive Predictive Value', ylab='Sensitivity')
legend(0,1,legend=an,pch=1:length(mat[1,]), col=1:length(mat[1,]),cex=1)
grid();
dev.off();



##Scatter plots
mat<-as.matrix(read.csv('mat.csv',header=FALSE, sep=','))
colnames(mat) <- an
postscript('../DisplayItems/cor1.eps', horizontal=FALSE)
x <- mat[,c(3,4)]
plot(x, main=sprintf('Spearman Correlation: %.2f', cor(x, method='spearman')[1,2]), axes=FALSE);
axis(1); axis(2);
dev.off();
postscript('../DisplayItems/cor2.eps', horizontal=FALSE)
x <- mat[,c(6,9)]
plot(x, main=sprintf('Spearman Correlation: %.2f', cor(x, method='spearman')[1,2]), axes=FALSE);
axis(1); axis(2);
dev.off();




##Print the ablation analysis table
an=c('BL*', 'CG*', 'T99', 'CONTRAFold1.1', 'DIM-CG', 'NOM-CG', 'BL-FR*', 'MaxExpect', 'CONTRAFold2.0', 'CentroidFold', 'Threshold', 'Train', 'Test')
mat<-as.matrix(read.csv('abmat.csv',header=FALSE, sep=','))
colnames(mat) <- ((1:length(mat[1,])-1))
mat[which(mat<0)] <- 0
tmp <- vector();
for (i in 1:(length(an)-3))
  tmp[i] <- length(which(is.nan(mat[i,])))

algo.inds <- sort(tmp, decreasing=TRUE, index.return=TRUE)$ix

mat[1:(length(an)-3),] <- mat[algo.inds,]
rownames(mat) <- c(an[algo.inds], an[(length(an)-2):length(an)])
mat[which(is.nan(mat))] <- ''
tmp <- matrix(as.numeric(mat),dim(mat)[1], dim(mat)[2])
rownames(tmp) <- c(an[algo.inds], an[(length(an)-2):length(an)])
sink(file='../DisplayItems/AblationAnalysis.tex', append=FALSE, 'output', FALSE);
print(xtable(tmp, digits=4))
sink(NULL)
