/*
 * A header file for a class that calculates a dot plot and writes image output
 * to a Postscript file.
 * This class can also write a dot plot text file.
 *
 * (c) 2009  Mathews Lab, University of Rochester Medical Center
 * Written by Jessica Reuter
 */

#include "../src/DotPlotHandler.h"
#include "../src/ParseCommandLine.h"
#include "../src/PostscriptWriting/Postscript_Wrapper.h"

using namespace std;

class DotPlots {
 public:
  // Public constructor and methods

  /*
   * Name:        Constructor
   * Description: Initalizes all private variables
   */
  DotPlots();

  /*
   * Name:        parse
   * Description: Parses command line arguments to determine what options are
   *              required to create a particular dot plot.
   * Arguments:
   *     1.   The number of command line arguments
   *     2.   The command line arguments themselves
   * Returns:
   *     true if parsing completed without errors, false if not
   */
  bool parse( int argc, char* argv[] );

  /*
   * Name:        run
   * Description: Run dot plot creation.
   */
  void run();

 private:
  /*
   * Name:        identify
   * Description: Identify the name of the executable, used to distinguish a
   *              requested dot plot type.
   * Argument:
   *         The raw executable name, possibly with path
   * Returns:
   *         The name of the executable.
   */
  string identify( string executableName );

 private:
  // Private variables.

  // Dot plot handler
  DotPlotHandler* plotMaker;

  // Boolean, true if a plot text file is written, false if a Postscript image
  // is written
  bool simpleOut;
};
