/*
 * A header file for a program that finds pseudoknots in a strand of nucleic
 * acids.
 * This strand of nucleic acids can be composed of either DNA or RNA.
 *
 * (c) 2010  Mathews Lab, University of Rochester Medical Center
 * Written by Jessica Reuter
 */

#ifndef PROBKNOT_H
#define PROBKNOT_H

#include "../RNA_class/RNA.h"
#include "../src/ErrorChecker.h"
#include "../src/ParseCommandLine.h"

class ProbKnot {
 public:
  // Public constructor and methods

  /*
   * Name:        Constructor
   * Description: Initalizes all private variables
   */
  ProbKnot();

  /*
   * Name:        parse
   * Description: Parses command line arguments to determine what options are
   *              required for a particular ProbKnot calculation.
   * Arguments:
   *     1.   The number of command line arguments
   *     2.   The command line arguments themselves
   * Returns:
   *     true if parsing completed without errors, false if not
   */
  bool parse( int argc, char** argv );

  /*
   * Name:        run
   * Description: Run ProbKnot calculations.
   */
  void run();

 private:
  // Input and output file names
  char* inFile;            // The input file
  char* ctFile;            // The output ct file

  // Boolean true/false flags
  bool isSequence;           // Is input a sequence file (true) or not (false)?
  bool isRNA;                // Is input RNA (true) or DNA (false)

  // The number of iterations for the algorithm to do.
  int iterations;

  // The minimum helix length allowed.
  int minHelixLength;
};

#endif /* PROBKNOT_H */
