/*
 * A header file for a program that creates structures based on levels of
 * probable pairs.
 *
 * (c) 2009  Mathews Lab, University of Rochester Medical Center
 * Written by Jessica Reuter
 */

#ifndef PROBABLE_PAIRS_H
#define PROBABLE_PAIRS_H

#include "../RNA_class/RNA.h"
#include "../src/ErrorChecker.h"
#include "../src/ParseCommandLine.h"

class ProbablePair {
 public:
  // Public constructor and methods

  /*
   * Name:        Constructor
   * Description: Initalizes all private variables
   */
  ProbablePair();

  /*
   * Name:        parse
   * Description: Parses command line arguments to determine what options are
   *              required for probable pair structure determination.
   * Arguments:
   *     1.   The number of command line arguments
   *     2.   The command line arguments themselves
   * Returns:
   *     true if parsing completed without errors, false if not
   */
  bool parse( int argc, char** argv );

  /*
   * Name:        run
   * Description: Run probable pair structure generation
   */
  void run();

 private:
  // Input and output file names
  char* input;             // The input pfs file
  char* output;            // The output ct file

  // Float designating what threshold should be used
  float threshold;
};

#endif /* PROBABLE_PAIR_H */
