/*
 * A header file for a program that calculates the free energy of a strand of
 * nucleic acids.
 * This strand of nucleic acids can be composed of either DNA or RNA.
 *
 * (c) 2008  Mathews Lab, University of Rochester Medical Center
 * Written by Jessica Reuter
 */

#ifndef EFN2_H
#define EFN2_H

#include <iomanip>
#include <vector>

#include "../RNA_class/RNA.h"
#include "../src/ErrorChecker.h"
#include "../src/ParseCommandLine.h"

class efn2Interface {
 public:
  // Public constructor and methods

  /*
   * Name:        Constructor
   * Description: Initalizes all private variables
   */
  efn2Interface();

  /*
   * Name:        parse
   * Description: Parses command line arguments to determine what options are
   *              required for a particular efn2 calculation.
   * Arguments:
   *     1.   The number of command line arguments
   *     2.   The command line arguments themselves
   * Returns:
   *     true if parsing completed without errors, false if not
   */
  bool parse( int argc, char** argv );

  /*
   * Name:        run
   * Description: Run efn2 calculations and output in the manner specified
   */
  void run();

 private:
  // Private variables

  // True/false boolean flags
  bool isRNA;              // Is efn2 analysis of RNA?
  bool writeTherm;         // Write a thermodynamic details file?
  bool stdPrint;           // Print to standard output?

  // Input and output file names
  char* ctFile;            // The input ct file
  char* outFile;           // The output file

  // Temperature at which calculations occur
  float temperature;

  // The number of structures in the CT file.
  int structures;

  // The free energy values calculated.
  vector<double> energies;
};

#endif /* EFN2_H */
