/*
 * A header file for a program that folds a strand of nucleic acids.
 * This strand of nucleic acids can be composed of either DNA or RNA.
 *
 * (c) 2009  Mathews Lab, University of Rochester Medical Center
 * Written by Jessica Reuter
 */

#ifndef FOLD_H
#define FOLD_H

#include "../RNA_class/RNA.h"
#include "../src/ErrorChecker.h"
#include "../src/ParseCommandLine.h"

class Fold {
 public:
  // Public constructor and methods

  /*
   * Name:        Constructor
   * Description: Initalizes all private variables
   */
  Fold();

  /*
   * Name:        parse
   * Description: Parses command line arguments to determine what options are
   *              required for a particular fold calculation.
   * Arguments:
   *     1.   The number of command line arguments
   *     2.   The command line arguments themselves
   * Returns:
   *     true if parsing completed without errors, false if not
   */
  bool parse( int argc, char** argv );

  /*
   * Name:        run
   * Description: Run folding calculations
   */
  void run();

 private:
  // True/false boolean flag to determine if fold analysis is of RNA
  bool isRNA;

  // Input and output file names
  char* seqFile;                 // The input sequence file
  char* ctFile;                  // The output ct file
  const char* saveFile;          // The output save file

  const char* constraintFile;    // The folding constraints file
  const char* SHAPEFile;         // The SHAPE constraints file

  const char* singleOffsetFile;  // The single strand offset file
  const char* doubleOffsetFile;  // The double strand offset file

  // Folding parameters
  int percent;                   // The percent difference
  int numStructures;             // The number of structures
  int windowSize;                // The window size

  int maxDistance;               // The maximum distance
  int maxLoop;                   // The maximum loop size

  double slope;                  // The slope for SHAPE constraints
  double slopeSingle;	         // The slope for single-stranded
                                 //   SHAPE constraints

  double intercept;              // The intercept for SHAPE constraints
  double interceptSingle;        // The intercept for single-stranded
                                 //   SHAPE constraints

  // The temperature at which folding occurs
  float temperature;
};

#endif /* FOLD_H */
