// This program uses dsv file and ct file to calculate the best maxdsvchange for the next pairwise progressive calculation.
// This file should be compiled in .../RNAstructure/dynalign/ directory
// Modified by ZJX 12/01/2008

#include <fstream>
#include <iostream>

#include "defines.h"
#include "dynalign.h"
#include "dynalignarray.h"
#include "dynalignheap.h"
#include "platform.h"
#include "rna_library.h"
#include "structure.h"
#include "varray.h"
#include "wendarray.h"
#include "algorithm.h"
#include "arrayclass.h"
#include "dynalignstackclass.h"
#include "dotarray.h"
#include "forceclass.h"
#include "observable.h"
#include "stackclass.h"
#include "TProgressDialog.h"
#include "errmsg.h"
#include "configfile.h"
#include "outputconstraints.h"
#include "observer.h"
#include "observingtextprogressbar.h"
#include "stackstruct.h"

using namespace std;

#define maxfilelength 100
#define INTERVAL 0.5

void calculatedots(char *savefilename, char *ctfilename, char *basepr, double maxpercent) {
	
  datatable *data;
  //ZJX10032008:minimum free energy;"energy" stores lowest free energy to allow all the true base pairs;"lowestij" stores lowest energy for ij bp;
  short mfg, energy, lowestij; 
	
  //ZJX10032008:the best dsvchange to predict a most correct secondary structure
  long double bestdsvchange;
  
  short i,j,k,l,crit,ir,en1,*lowend,*highend;
  bool singleinsert,local;
  short maxsep,gap,lowest;
	
  dynalignarray *w,*vmod;
  varray *v;
  wendarray *w3,*w5;
	
  short int inc[6][6]={{0,0,0,0,0,0},{0,0,0,0,1,0},{0,0,0,1,0,0},{0,0,1,0,1,0},
                       {0,1,0,1,0,0},{0,0,0,0,0,0}};
  int modificationflag;

  bool optimalonly;
  bool **allowed_alignments;

  //create and open basepr file for input
  ofstream outbp(basepr);
  if(!outbp.is_open())
  {
      cerr << "basepr can't be created and opened.\n";
  }

  structure *ct1, *ct2, *knownct;
  ct1 = new structure();
  ct2 = new structure();
  knownct = new structure();
  openct(knownct, ctfilename);

  data = new datatable();

  //open the save file to peek at the sizes needed to allocate arrays:
  ifstream sav(savefilename,ios::binary);
  //note that this includes modification and optimal only data
  read(&sav, &modificationflag); 

  if (modificationflag==2||modificationflag==3) optimalonly=true;
  else optimalonly=false;

  //start with structure information
  read(&sav,&(ct1->numofbases));
  read(&sav,&(ct2->numofbases));
  read(&sav,&maxsep);
  sav.close();
	
  if (maxsep<0) {
	  //This means that the allowed alignments were constrained by the HMM alignment
	  //probs and not by the traditional M parameter.
	  //Therefore, space must be allocated for the allowed_alignment arrays.

	  allowed_alignments=new bool *[ct1->numofbases+1];
	  for (i=0;i<=ct1->numofbases;i++) allowed_alignments[i]=new bool [ct2->numofbases+1];

  }
  else allowed_alignments=NULL;
  //fill the low and highend arrays:
  //allocate space:
  lowend = new short [2*ct1->numofbases];
  highend = new short [2*ct1->numofbases];
	
  //DHM (11/30/06): move these allocations to where the file is read
  if (modificationflag==1||modificationflag==3) {
    vmod = new dynalignarray();
  }
  else vmod=NULL;

  v = new varray();
  w = new dynalignarray();


  if (!optimalonly) w3 = new wendarray();
  else w3= NULL;
  w5 = new wendarray();


  //open the save file
  opendynalignsavefile(savefilename, ct1, ct2, v, w, vmod, w3, w5, data, &singleinsert,  &maxsep, &gap, &lowest,&local,allowed_alignments,lowend,highend);
  

  //double up the sequences as part of suboptimal traceback support:
  for (i=1;i<=ct1->numofbases;i++) ct1->numseq[(ct1->numofbases)+i] = ct1->numseq[i];
  for (i=1;i<=ct2->numofbases;i++) ct2->numseq[(ct2->numofbases)+i] = ct2->numseq[i];


  crit = DYNALIGN_INFINITY;
  for (i=1;i<=ct1->numofbases;i++) {
	for (k=max(lowend[i],1);k<=min(highend[i],ct2->numofbases);k++) {
		if (local) { 
			if ((w5->f(i,k))<crit) {
			    crit = w5->f(i,k);
			}
		}
			else { 
				if ((w5->f(i,k)+gap*abs(ct1->numofbases-i-(ct2->numofbases-k))  )<crit) {
				    crit = w5->f(i,k)+gap*abs(ct1->numofbases-i-(ct2->numofbases-k));
				}
			}	
	}
  }
  
  //ZJX10032008
  mfg = crit;//the minimum of the total free energy of the 2 sequences
  
  crit = crit - (short) (integersize)((float) crit*(((float) maxpercent)/100.0 ));

  //ZJX10032008

  outbp << "i\t" << "j\t" <<  "k\t" << "l\t" << "lowest_energy_containing_i-j\t" << "percent_above_mfg\n";
  int k0=0, l0=0;
  long double percentage;
  energy = mfg;
  unsigned int categories = 100/INTERVAL;
  unsigned int category_num;
  int *count = new int [categories]();//initialized to zero
  int *count2 = new int [categories]();//initialized to zero
// for(i=0; i!= categories; ++i)
//  cout << *(count+i) << " " << *(count2+i) << endl;
  int bigger=0;
  int bigger2=0;
  for (i=1;i <= ct1->numofbases;i++) {
     for (j=i+minloop;j<=ct1->numofbases;j++){
     
         lowestij = DYNALIGN_INFINITY;  
         int knownct_j=knownct->basepr[1][i];
         
         for (k=max(lowend[i],1);k<=(min(ct2->numofbases,highend[i]));k++)
            for (l=max(lowend[j],k);l<=(min(ct2->numofbases,highend[j]));l++) {
                
                int sumofv = v->f(i,j,k,l)+v->f(j,i+ct1->numofbases,l,k+ct2->numofbases);
                if (sumofv <= lowestij) { //find lowest energy for every possible base pair
                      lowestij = sumofv;
	                    k0=k;
                      l0=l;
                }
            } 
            
         percentage = (long double)100*(mfg-lowestij)/(long double)mfg;
         if(percentage < 100){
             category_num = percentage / INTERVAL;
            ++(*(count+category_num));
         }
         else  bigger++;
      
         if(knownct_j==j && j>i){
              outbp << i << "\t" << j << "\t" << k0 << "\t" << l0 << "\t" << lowestij << "\t\t" << percentage << "\n";
              if(percentage < 100){
                 category_num = percentage / INTERVAL;
                 ++(*(count2+category_num));
              }
              else  bigger2++;
              if(lowestij >= energy && lowestij < crit)
                 energy=lowestij;
         }
      
    }

  }
  outbp << "\n###########################################################################\n" << endl;
  bestdsvchange = 100 * (long double)(mfg-energy)/(long double) (mfg);//energy is 10*(absolute maxdsvchange value[not percentage])
  cout << "maxdsvchange = " << bestdsvchange <<"\n";
  outbp << "# The minimal free energy * 10: " << mfg << "\n";
  outbp << "# The best maxdsvchange parameter is: " << bestdsvchange << "(percentage)\t" << (double)(mfg-energy)/(double)10 << "(absolute value)" << "\n";  
//  cout << "maxdsvchange = " << bestdsvchange <<"\n"; // this standard output is supposed to be redirected to the configuration files

  unsigned int total=0, total2=0;
  outbp << "\n###########################################################################\n" << endl
        << "# The energy distribution of lowestij of i-j pairs in the " << savefilename << ":" << endl
        << "Percentage_above_mfg\tPair_Count(all_i-j_pairs)\tPair_Count(true_i-j_pairs)" << endl;
  for(int i=0; i != categories; ++i){
    outbp << "[" << i*INTERVAL << "    " << (i+1)*INTERVAL << ")\t\t" << *(count+i) << "\t\t" << *(count2+i) << endl;
    total += *(count+i);
    total2 += *(count2+i);
  }
  outbp << "[100    +)\t\t" << bigger << "\t\t" << bigger2 << endl;
  outbp << "Total_number:\t\t" << total + bigger << "\t\t" << total2 + bigger2 << endl;

 // cout << total <<" " << ct1->numofbases << endl;
//  unsigned int total2=0;
//  outbp << "\n##########################################################################\n" << endl
//        <<"# The energy distribution of lowestij of true basepairs:" << endl
//        << "Percentage above mfg\tPair Count" << endl ;
//  for(int i=0; i != categories; ++i){
//    outbp << "[" << i*INTERVAL << "\t" << (i+1)*INTERVAL << ")\t\t" << *(count2+i) << endl;
//    total2 += *(count2+i);
//  }
//  outbp << "[100\t+)\t\t" << bigger2 << endl;
//  outbp << "# Total number of true basepairs: " << total2 + bigger2 << endl;
  

  delete w3;
  delete w5;
  delete w;
  delete v;

  if (maxsep<0) {
	  
		for (i=0;i<=ct1->numofbases;i++) delete[] allowed_alignments[i];
		delete[] allowed_alignments;
  }

  if (modificationflag==1||modificationflag==3) delete vmod;

  delete data;
  delete ct1;
  delete ct2;
  delete knownct;
  delete[] lowend;
  delete[] highend;
  delete [] count;
  delete [] count2;

  outbp.close();
}


int main(int argc, char* argv[])
{
  char savefile[maxfilelength],ctfile[maxfilelength],basepr[maxfilelength];
  double percent;

  if (argc!=5) 
  {
    cout << "command line usage: bestdsvchange input.dsv max_%_difference known.ct output.bp\n\n";
    exit(0);
  }
  else 
  {
    strcpy(savefile,argv[1]);
    percent=atof(argv[2]);
    strcpy(ctfile,argv[3]);
    strcpy(basepr,argv[4]);
  }

  calculatedots(savefile,ctfile,basepr,percent);

  return 0;
}
