#include <fstream>
#include <iostream>
#include <cstring>
#include <cstdlib>
#include <vector>
#include <string>
#include "../src/configfile.h"
#include "../src/defines.h"
#include "../src/dynalign.h"
#include "../src/dynalignheap.h"
#include "../src/outputconstraints.h"
#include "../src/platform.h"
#include "../src/rna_library.h"
#include "../src/structure.h"
#include "../src/TProgressDialog.h"
#include "../src/align_services.h"
#include "../src/hmm_arrays.h"
#include "../RNA_class/Dynalign_object.h"
#include "../RNA_class/Multilign_object.hpp"
#ifdef DYNALIGN_SMP
#include "../src/observingtextprogressbar.h"
#endif

using namespace std;

int main(int argc, char* argv[]) {

  string inseq, outct, aout;
  // define default values for the parameters
  short int bpwin = 2, awin = 1, imaxseparation = -99, numProcessors = 1, cycles = 2, maxtrace=750, percent = 20, singlefold_subopt_percent = 30;
  size_t main_seq = 1;
  int maxpairs = -1;
  bool random = false, ali = false, dsv = true, ct_templated = false, insert = true, optimalonly = false, local = false;
  vector<string> seq_list;
  vector<string> ct_list;
  float maxdsvchange = 1;
  double fgap = 0.4;

  string constraint1;
  string constraint2;
  string constrainta;


  if( argc == 2 ) {

    ConfigFile config(argv[1]);
    // Check for mandatory config file settings.
    bool valid_config = config.contains("inseq") && config.contains("outct");

#ifdef DYNALIGN_SMP
    valid_config = valid_config && config.contains("num_processors");
#endif

    if (valid_config) {
      // Read all mandatory config file settings.
      inseq = config.getOption<string>("inseq");
      outct = config.getOption<string>("outct");

#ifdef DYNALIGN_SMP
      numProcessors = config.getOption<int>("num_processors");
#endif
    }
    char separator=';';
    inseq = inseq.substr( inseq.find('{')+1, inseq.rfind('}')-1 );
		outct = outct.substr( outct.find('{')+1, outct.rfind('}')-1 );
    istringstream issseq(inseq), issct(outct);
    string line;
    while ( getline(issseq, line, separator)){
      seq_list.push_back(line);
    }

    while ( getline(issct, line, separator)){
      ct_list.push_back(line);
    }
    cout << "Set seq_list and ct_list:\n";
    for(int i=0; i < seq_list.size(); ++i){
      cout << '\t' << seq_list[i] << '\t' << ct_list[i] << endl;
    }

    // Read optional config file settings
    if (valid_config) {
      //check to see if a ct file is specified for sequence 1, i.e. ct_templated = true.
      if (config.contains("ct_templated")) ct_templated = config.getOption<bool>("ct_templated");
      if (config.contains("imaxseparation")) imaxseparation = config.getOption<short int>("imaxseparation");
      if (config.contains("fgap")) fgap = config.getOption<double>("fgap");
      if (config.contains("maxtrace")) maxtrace = config.getOption<short int>("maxtrace");
      if (config.contains("percent")) percent = config.getOption<short int>("percent");
      if (config.contains("singlefold_subopt_percent")) singlefold_subopt_percent = config.getOption<short int>("singlefold_subopt_percent");
      if (config.contains("bpwin")) bpwin = config.getOption<short int>("bpwin");
      if (config.contains("awin")) awin = config.getOption<short int>("awin");
      if (config.contains("insert")) insert = config.getOption<bool>("insert");
      if (config.contains("constraint_1_file"))  constraint1 = config.getOption<string>("constraint_1_file");
      if (config.contains("constraint_2_file")) constraint2 = config.getOption<string>("constraint_2_file");
      if (config.contains("constraint_align_file"))  constrainta = config.getOption<string>("constraint_align_file");
      if (config.contains("optimal_only")) 	optimalonly = config.getOption<bool>("optimal_only");
      if (config.contains("local")) local = config.getOption<bool>("local");
      //multilign specific parameters:
      if (config.contains("cycles")) cycles = config.getOption<short int>("cycles");
      if (config.contains("maxpairs")) maxpairs = config.getOption<int>("maxpairs");
      if (config.contains("main_seq")) main_seq = config.getOption<size_t>("main_seq");
      if (config.contains("maxdsvchange")) maxdsvchange = config.getOption<float>("maxdsvchange");
      if (config.contains("random")) random = config.getOption<bool>("random");    
      if (config.contains("ali")) ali = config.getOption<bool>("ali");
      if (config.contains("dsv")) dsv = config.getOption<bool>("dsv");
    }
    valid_config = valid_config && (main_seq > 0) && (cycles > 0);
    if (!valid_config){
      cerr << "ERROR: At least one parameter could not be read from the "
        << "configuration file. Aborting." << endl;
      exit(1);
    }
  }
  else {
    // Interactive mode:
    cout << "Usage: multilign [config file]\nNo config file specified, using interactive mode:\n\n";
    unsigned i=0;
    bool query;
    string seq, ct;
    while(cin){
      
      cout << "Enter the name of sequence " << ++i << "  (enten Ctr+D to end): ";
      cin >> seq;
      seq_list.push_back(seq);
      
      cout << "Enter the name of output ct file for sequence " << i << ": ";
      cin >> ct;
              //        if(ct.empty)
              //          if( seq.rfind(".seq") == (seq.length() - 4))
              //            ct = seq.substr(0, seq.length()-4) + ".ct";
      ct_list.push_back(ct);
    }
    cin.clear();
    cout << "Enter name for the output of the alignment: ";
    cin >> aout;
    cout << "Enter the max separation (M) <Recommend enter -99 for constraint by HMM forward-backward>: ";
    cin >> imaxseparation;
    cout << "Enter the gap penalty: <Recommend 0.4>";
    cin >> fgap;
    cout << "Enter the maximum number of structures: ";
    cin >> maxtrace;
    cout << "Enter the maximum percent energy difference "
	 << "(where entering 20 is 20%): ";
    cin >> percent;
    cout << "Enter the maximum percent energy difference for keeping "
	 << "suboptimal structures from the single-sequence folding "
	 << "calculations: <Recommend 30>";
    cin >> singlefold_subopt_percent;
    cout << "Enter the base pair window size: <Recommend 2>";
    cin >> bpwin;
    cout << "Enter the alignment window size: <Recommend 1>";
    cin >> awin;
    cout << "Allow single BP inserts into one sequence? (1/0) <Recommend 1>";
    cin >> insert;
    cout << "Input constraints for sequence 1? (1/0) ";
    cin >> query;
    if (query) {
      cout << "Enter the constraint file name: ";
      cin >> constraint1;
    }
    cout << "Input constraints for sequence 2? (1/0) ";
    cin >> query;
    if (query) {
      cout << "Enter the constraint file name: ";
      cin >> constraint2;
    }
    cout << "Input constraints for alignment? (1/0) ";
    cin >> query;
    if (query) {
      cout << "Enter the constraint file name: ";
      cin >> constrainta;
    }
#ifdef DYNALIGN_SMP
    cout << "Enter the number of processors on this machine: ";
    cin >> numProcessors;
#endif
    //multilign specific:
    cout << "Enter the order number of one sequence to be main sequence(default is the 1st sequence): ";
    cin >> main_seq;
    cout << "Enter the percentage above the mini free energy for dsv templating <recommend 1>: ";
    cin >> maxdsvchange;
    cout << "Enter the number of the cycles to run through the calculation <recommend 2>: ";
    cin >> cycles;
    cout << "Enter the number of the basepairs to allow for dsv templating <recommend -1, indicating the length of the main sequence: ";
    cin >> maxpairs;
    cout << "Randomize the sequence order? Enter 0 (not to randomzie) or 1 (randomize): ";
    cin >> random;
    cout << "Output pairwise alignment? Enter 0 (not to output) or 1 (output): ";
    cin >> ali;
    cout << "Output pairwise dsv files? Enter 0 (not to output) or 1 (output): ";
    cin >> dsv;
  }
  
  Multilign_object a(seq_list, ct_list, random, dsv, ali);
  //cout << maxtrace << bpwin << awin << percent << imaxseparation << fgap << insert << optimalonly << singlefold_subopt_percent << local << numProcessors<< maxpairs<< maxdsvchange<< cycles << endl;
  if (a.GetErrorCode()){
    a.GetErrorMessage(a.GetErrorCode());
    exit(1);
  }
  a.calculate_dynalign(maxtrace, bpwin, awin, percent, imaxseparation, fgap, insert, optimalonly, singlefold_subopt_percent, local, numProcessors, maxpairs, maxdsvchange, cycles);
  if (a.GetErrorCode()){
    a.GetErrorMessage(a.GetErrorCode());
    exit(1);
  }
  cout << "Done!!!\n";
  return 0;
}
