// This program uses dsv file and predicted ct file to check the potential bugs 
// of conflick between w5 array and dot plot energy.
// Modified by ZJX 10/29/2009

#include <fstream>
#include <iostream>

#include "../src/defines.h"
#include "../src/dynalign.cpp"
#include "../src/dynalignarray.cpp"
#include "../src/dynalignheap.cpp"
#include "../src/platform.h"
#include "../src/rna_library.cpp"
#include "../src/structure.cpp"
#include "../src/varray.cpp"
#include "../src/wendarray.cpp"
#include "../src/algorithm.cpp"
#include "../src/arrayclass.cpp"
#include "../src/dynalignstackclass.cpp"
#include "../src/dotarray.cpp"
#include "../src/forceclass.cpp"
#include "../src/observable.cpp"
#include "../src/stackclass.cpp"
#include "../src/TProgressDialog.cpp"
#include "../src/errmsg.cpp"
#include "../src/configfile.cpp"
#include "../src/outputconstraints.cpp"
#include "../src/observer.cpp"
#include "../src/observingtextprogressbar.cpp"
#include "../src/stackstruct.cpp"

using namespace std;

#define maxfilelength 100
#define strucNum 1

void calculatedots(char *savefilename, char *ctfilename, int maxpercent) {
	
  datatable *data;
  //ZJX10032008:minimum free energy;"energy" stores lowest free energy to allow all the true base pairs;"lowestij" stores lowest energy for ij bp;
  short mfg, energy, lowestij; 
	
  //ZJX10032008:the best dsvchange to predict a most correct secondary structure
  long double bestdsvchange;
  
  short i,j,k,l,crit,ir,en1,*lowend,*highend;
  bool singleinsert,local;
  short maxsep,gap,lowest;
	
  dynalignarray *w,*vmod;
  varray *v;
  wendarray *w3,*w5;
	
  short int inc[6][6]={{0,0,0,0,0,0},{0,0,0,0,1,0},{0,0,0,1,0,0},{0,0,1,0,1,0},
                       {0,1,0,1,0,0},{0,0,0,0,0,0}};
  int modificationflag;

  bool optimalonly;
  bool **allowed_alignments;

  structure *ct1, *ct2, *ct;
  ct1 = new structure();
  ct2 = new structure();
  ct = new structure();
  openct(ct, ctfilename);

  data = new datatable();
  
  //open the save file to peek at the sizes needed to allocate arrays:
  ifstream sav(savefilename,ios::binary);
  //note that this includes modification and optimal only data
  read(&sav, &modificationflag); 
  
  if (modificationflag==2||modificationflag==3) optimalonly=true;
  else optimalonly=false;
  
  //start with structure information
  read(&sav,&(ct1->numofbases));
  read(&sav,&(ct2->numofbases));
  read(&sav,&maxsep);
  sav.close();
  
  if (maxsep<0) {
    //This means that the allowed alignments were constrained by the HMM alignment
    //probs and not by the traditional M parameter.
    //Therefore, space must be allocated for the allowed_alignment arrays.
    
    allowed_alignments=new bool *[ct1->numofbases+1];
    for (i=0;i<=ct1->numofbases;i++) allowed_alignments[i]=new bool [ct2->numofbases+1];
    
  }
  else allowed_alignments=NULL;
  //fill the low and highend arrays:
  //allocate space:
  lowend = new short [2*ct1->numofbases];
  highend = new short [2*ct1->numofbases];
  
  //DHM (11/30/06): move these allocations to where the file is read
  if (modificationflag==1||modificationflag==3) {
    vmod = new dynalignarray();
  }
  else vmod=NULL;
  
  v = new varray();
  w = new dynalignarray();
  
  if (!optimalonly) w3 = new wendarray();
  else w3= NULL;
  w5 = new wendarray();
  
  //open the save file
  opendynalignsavefile(savefilename, ct1, ct2, v, w, vmod, w3, w5, data, &singleinsert,  &maxsep, &gap, &lowest,&local,allowed_alignments,lowend,highend);
  
  //double up the sequences as part of suboptimal traceback support:
  for (i=1;i<=ct1->numofbases;i++) ct1->numseq[(ct1->numofbases)+i] = ct1->numseq[i];
  for (i=1;i<=ct2->numofbases;i++) ct2->numseq[(ct2->numofbases)+i] = ct2->numseq[i];
  
  crit = DYNALIGN_INFINITY;
  for (i=1;i<=ct1->numofbases;i++) {
    for (k=max(lowend[i],1);k<=min(highend[i],ct2->numofbases);k++) {
      if (local) { 
	if ((w5->f(i,k))<crit) {
	  crit = w5->f(i,k);
	}
      }
      else { 
	if ((w5->f(i,k)+gap*abs(ct1->numofbases-i-(ct2->numofbases-k))  )<crit) {
	  crit = w5->f(i,k)+gap*abs(ct1->numofbases-i-(ct2->numofbases-k));
	}
      }	
    }
  }
  
  //at this point crit is the minimum of the total free energy of the 2 sequences
 
  //ZJX10032008
  int k0=0, l0=0;
  bool flag=true;
  for (i=1;i <= ct1->numofbases;i++) {
    j = ct->basepr[1][i];
   
    if (j > i) {

      lowestij = DYNALIGN_INFINITY; 
      for (k=max(lowend[i],1);k<=(min(ct2->numofbases,highend[i]));k++)
	for (l=max(lowend[j],k);l<=(min(ct2->numofbases,highend[j]));l++) {	  
	  int sumofv = v->f(i,j,k,l)+v->f(j,i+ct1->numofbases,l,k+ct2->numofbases);
	  if (sumofv <= lowestij) { //find lowest energy for every possible base pair
	    lowestij = sumofv;
	    k0=k;
	    l0=l;
	  }
	}
      if ( lowestij != crit ){           
	if (flag){
	  cerr << "========= " << savefilename << " ==================\n";
	  cerr << "i\tj\tk\tl\tmfg_with_ij\tMFE\n";
	  flag = false;
	}
	cerr << i << "\t" << j << '\t' << k0 << '\t' << l0 << '\t' << lowestij << '\t' << crit << endl;
      }
      
    }
  }
    //  unsigned int total2=0;
//  outbp << "\n##########################################################################\n" << endl
//        <<"# The energy distribution of lowestij of true basepairs:" << endl
//        << "Percentage above mfg\tPair Count" << endl ;
//  for(int i=0; i != categories; ++i){
//    outbp << "[" << i*INTERVAL << "\t" << (i+1)*INTERVAL << ")\t\t" << *(count2+i) << endl;
//    total2 += *(count2+i);
//  }
//  outbp << "[100\t+)\t\t" << bigger2 << endl;
//  outbp << "# Total number of true basepairs: " << total2 + bigger2 << endl;

  delete w3;
  delete w5;
  delete w;
  delete v;

  if (maxsep<0) {
		for (i=0;i<=ct1->numofbases;i++) delete[] allowed_alignments[i];
		delete[] allowed_alignments;
  }

  if (modificationflag==1||modificationflag==3) delete vmod;

  delete data;
  delete ct1;
  delete ct2;
  delete ct;
  delete[] lowend;
  delete[] highend;

}


int main(int argc, char* argv[])
{
  char savefile[maxfilelength],ctfile[maxfilelength];
  int percent;

  if (argc < 3) 
  {
    cout << "command line usage: maxdsvchange input.dsv (max_%_difference) predicted.ct\n";
    exit(1);
  }
  else if (argc == 4)
  {
    strcpy(savefile,argv[1]);
    percent=atof(argv[2]);
    strcpy(ctfile,argv[3]);
  }
  else{
    strcpy(savefile,argv[1]);
    percent = 80;
    strcpy(ctfile,argv[2]);
  }
  
  calculatedots(savefile,ctfile,percent);
  
  return 0;
}
