/*
 * A header file for a program that calculates thermodynamic parameters from a
 * set of oligonucleotides.
 *
 * (c) 2009  Mathews Lab, University of Rochester Medical Center
 * Written by Jessica Reuter
 */

#include "../RNA_class/Oligowalk_object.h"
#include "../src/ErrorChecker.h"
#include "../src/ParseCommandLine.h"

using namespace std;

class oligoscreen {
 public:
  // Public constructor and methods

  /*
   * Name:        Constructor
   * Description: Initalizes all private variables
   */
  oligoscreen();

  /*
   * Name:        parse
   * Description: Parses command line arguments to determine what options are
   *              required for a particular OligoScreen calculation.
   * Arguments:
   *     1.   The number of command line arguments
   *     2.   The command line arguments themselves
   * Returns:
   *     true if parsing completed without errors, false if not
   */
  bool parse( int argc, char** argv );

  /*
   * Name:        run
   * Description: Run OligoScreen calculations
   */
  void run();

 private:
  // Private usage method
  /*
   * Name:        usage
   * Description: Prints a usage message with option descriptions.
   */
  void usage();

 private:
  // Input and output file names
  char* listFile;          // The input oligo list file
  char* reportFile;        // The output report file

  // Boolean to show if RNA chemistry should be used
  bool isRNA;
 
  // The temperature at which thermodynamics are calculated
  float temperature;
};
