/*
 * A header file for a program that calculates the partition function for a
 * strand of nucleic acids.
 * This strand of nucleic acids can be composed of either DNA or RNA.
 *
 * (c) 2009  Mathews Lab, University of Rochester Medical Center
 * Written by Jessica Reuter
 */

#ifndef PARTITION_H
#define PARTITION_H

#include "../RNA_class/RNA.h"
#include "../src/ErrorChecker.h"
#include "../src/ParseCommandLine.h"

class partitionInterface {
 public:
  // Public constructor and methods

  /*
   * Name:        Constructor
   * Description: Initalizes all private variables
   */
  partitionInterface();

  /*
   * Name:        parse
   * Description: Parses command line arguments to determine what options are
   *              required for a particular partition function calculation.
   * Arguments:
   *     1.   The number of command line arguments
   *     2.   The command line arguments themselves
   * Returns:
   *     true if parsing completed without errors, false if not
   */
  bool parse( int argc, char** argv );

  /*
   * Name:        run
   * Description: Run partition function calculation
   */
  void run();

 private:
  // Input and output file names
  char* seqFile;                 // The input sequence file
  char* pfsFile;                 // The output partition function save file

  const char* constraintFile;    // The constraints file
  const char* SHAPEFile;         // The SHAPE constraints file

  // SHAPE parameters
  double slope;                  // The slope for SHAPE constraints
  double intercept;              // The intercept for SHAPE constraints

  // The maximum distance allowed between paired nucleotides
  int distance;

  // The temperature at which calculation occurs
  float temperature;

  // True/false boolean flag showing if DNA (false) or RNA (true) is analyzed
  bool isRNA;
};

#endif /* PARTITION_H */
