/*
 * A header file for a program that folds a strand of nucleic acids from a
 * previously folded save file.
 * This strand of nucleic acids can be composed of either DNA or RNA.
 *
 * (c) 2009  Mathews Lab, University of Rochester Medical Center
 * Written by Jessica Reuter
 */

#ifndef REFOLD_H
#define REFOLD_H

#include "../RNA_class/RNA.h"
#include "../src/ErrorChecker.h"
#include "../src/ParseCommandLine.h"

class refold {
 public:
  // Public constructor and methods

  /*
   * Name:        Constructor
   * Description: Initalizes all private variables
   */
  refold();

  /*
   * Name:        parse
   * Description: Parses command line arguments to determine what options are
   *              required for a particular fold calculation.
   * Arguments:
   *     1.   The number of command line arguments
   *     2.   The command line arguments themselves
   * Returns:
   *     true if parsing completed without errors, false if not
   */
  bool parse( int argc, char** argv );

  /*
   * Name:        run
   * Description: Run folding calculations
   */
  void run();

 private:
  // Input and output file names
  char* saveFile;          // The input save file
  char* ctFile;            // The output ct file

  // Folding parameters
  int percent;             // The percent difference
  int numStructures;       // The number of structures
  int windowSize;          // The window size
};

#endif /* REFOLD_H */
