#ifndef DOT_PLOT_HANDLER_H
#define DOT_PLOT_HANDLER_H

#include <limits>
#include <iostream>
#include <sstream>
#include <string>
#include <vector>

#include "ErrorChecker.h"
#include "../RNA_class/Dynalign_object.h"
#include "../RNA_class/RNA.h"

using namespace std;

class DotPlotHandler {
 public:
  DotPlotHandler( string inFile, string outFile );
  ~DotPlotHandler();
  string getColorAsRGB( const int i, const int j );
  string getColorAsPostscript( const int i, const int j );
  int getEntries();
  string getInfoString();
  string getInputFile();
  double getMaximum();
  double getMinimum();
  string getOutputFile();
  char getPlotType();
  double getValue( const int i, const int j );
  bool isError();
  void setEntries( int newEntries );
  void setMaximum( const double newMaximum );
  void setMinimum( const double newMinimum );
  void setPlotType( char type );
  void readDynalignSeq1Data();
  void readDynalignSeq2Data();
  void readFoldingData();
  void readPartitionData();
  void readTextData();
  void resetRange();
  void writePlotFile();

 public:
  static const int DEFAULT_ENTRIES = 5;
  static const int MIN_COLORS = 3;
  static const int MAX_COLORS = 15;

  static const char TYPE_DYNALIGN1 = '1';
  static const char TYPE_DYNALIGN2 = '2';
  static const char TYPE_ENERGY = 'e';
  static const char TYPE_PROBABILITY = 'p';
  static const char TYPE_UNDEFINED = 'X';

 private:
  void buildThresholds();
  void checkBounds();
  int getThresholdIndex( const int i, const int j );
  void initializeValuesArray( int length );
  void readDynalignData( int sequence );
  void readSingleStrandData( int type );

 private:
  string input;
  string output;
  string label;
  double defaultMax;
  double defaultMin;
  double maximum;
  double minimum;
  bool error;
  int entries;
  char plotType;
  string infoString;
  bool dataRead;
  vector< vector<double> > colorInfo;
  vector<double> thresholds;
  vector< vector<double> > values;
};

#endif /* DOT_PLOT_HANDLER_H */
