
#include "OligoScreenCalc.h"
#include "algorithm.h"
#include <cstring>
#include <fstream>

using namespace std;

//Perform the backend calculation for OligoScreen

//Return an int that indicates an error
//0 = no error

//Pre-define the suboptimal structure prediction parameters.
#define CNTR6 100 //tracebacks
#define CNTR8 20 //percent sort
#define CNTR9 0 //window size


int bimolecular(structure *ct,datatable *data) {
	//Calculate the bimolecular folding free energy
	structure *ct2;
	int i,energy;

	ct2 = new structure();
	

	

	//first create a structure with the sequence needed for bimolecular folding
	ct2->allocate(2*ct->numofbases+3);
	ct2->intermolecular = true;
	for (i=1;i<=ct->numofbases;i++) {
		ct2->numseq[i] = ct->numseq[i];
		ct2->numseq[i+ct->numofbases+3] = ct->numseq[i];


	}

	ct2->numseq[ct->numofbases+1] = 5;
	ct2->numseq[ct->numofbases+2] = 5;
	ct2->numseq[ct->numofbases+3] = 5;

	ct2->inter[0] = ct->numofbases+1;
	ct2->inter[1] = ct->numofbases+2;
	ct2->inter[2] = ct->numofbases+3;

	ct2->numofbases = 2*ct->numofbases+3;

	strcpy(ct2->ctlabel[1],ct->ctlabel[1]);
		
	//now fold and perform efn2
	dynamic(ct2,data,CNTR6,CNTR8,CNTR9,NULL,true);
	
	
	energy = ct2->energy[1];
	delete ct2;
	return energy;

}

//Perform the OligoScreen calculation.
int OligoScreenCalc(const char *infilename, const char *outfilename, datatable *data, rddata *hybriddata) {
	ifstream in;
	ofstream out;
	char temp[500];
	int i,j,k,l,energy;
	structure *ct;


	in.open(infilename);
	out.open(outfilename);
	
	out << "Sequence\tDGbimolecular\tDGunimolecular\tDGduplex\tDG2BPat5'\tDG2BPat3'\n";
	in >> temp;

	while (!in.eof()) {
		
		

		out << temp << "\t";
		//convert temp to a ct file
		i = strlen(temp);
		ct = new structure();

		ct->allocate(i);
		ct->numofbases=i;

		strcpy(ct->ctlabel[1],"");
		for (j=0;j<i;j++) {

			if (temp[j]=='A'||temp[j]=='a') ct->numseq[j+1] = 1;
			else if (temp[j]=='C'||temp[j]=='c') ct->numseq[j+1] = 2;
			else if (temp[j]=='G'||temp[j]=='g') ct->numseq[j+1] = 3;
			else  ct->numseq[j+1] = 4;

		}


		//First Calculate the Bimolecular folding DG:
		energy= bimolecular(ct,data);

		//gcvt((float (energy))/10,6,temp);
		if (conversionfactor==10) sprintf(temp,"%.1f",(float (energy))/conversionfactor);
		else sprintf(temp,"%.2f",(float (energy))/conversionfactor); //assume conversionfactor==100

		out << temp<<"\t";


		//Now do the unimolecular folding:
		dynamic(ct,data,CNTR6,CNTR8,CNTR9);
		//efn2 (&data,ct);--no efn2 needed with current parameters
		//sortstructures (ct);

		//gcvt((float (ct->energy[1]))/10,6,temp);
		if (conversionfactor==10) sprintf(temp,"%.1f",(float (ct->energy[1]))/conversionfactor);
		else sprintf(temp,"%.2f",(float (ct->energy[1]))/conversionfactor); //assume conversionfactor==100

		out << temp<<"\t";
		

		//now calculate the duplex free energy:
			
		//if (!pObject->isRNA) {//oligo is DNA:
		if (hybriddata!=NULL) {
      		energy = hybriddata->init;//initiation
			for (j=1;j<(ct->numofbases);j++) {
          		energy = energy + hybriddata->stack[complement(j+1,ct)][ct->numseq[j+1]][complement(j,ct)][ct->numseq[j]];
			}


		}

		else {//oligo is RNA:
      		energy = data->init;//initiation
      		for (j=1;j<(ct->numofbases);j++) {
          		energy = energy + data->stack[ct->numseq[j]]
            		[complement(j,ct)][ct->numseq[j+1]][complement(j+1,ct)];
			}


			//add end correction for RNA-RNA duplexes
			if (ct->numseq[1]==4||ct->numseq[1]==1) {
				energy = energy + data->auend;
			}
			if (ct->numseq[ct->numofbases]==4||ct->numseq[ct->numofbases]==1) {
				energy = energy + data->auend;
			}

		}

		//gcvt((float (energy))/10,6,temp);
		if (conversionfactor==10) sprintf(temp,"%.1f",(float (energy))/conversionfactor);
		else sprintf(temp,"%.2f",(float (energy))/conversionfactor); //assume conversionfactor==100


		out << temp<<"\t";

		//Added 1/11/05
		//Calculate the free energy cost of opening two base pairs at 5' end of duplex and 3' end of duplex
		
		//start at 5' end of sequence submitted
		
      	energy = 0;
		for (j=1;j<(ct->numofbases)&&j<3;j++) {
          	energy = energy - data->stack[ct->numseq[j]][complement(j,ct)][ct->numseq[j+1]][complement(j+1,ct)];
		}
		//check for AU/GU ends
		if (ct->numseq[1]==4||ct->numseq[1]==1) {
			energy = energy - data->auend;
		}
		if (ct->numseq[3]==4||ct->numseq[3]==1) {
			energy = energy + data->auend;
		}
		
		//gcvt((float (energy))/10,6,temp);
		if (conversionfactor==10) sprintf(temp,"%.1f",(float (energy))/conversionfactor);
		else sprintf(temp,"%.2f",(float (energy))/conversionfactor); //assume conversionfactor==100


		out << temp<<"\t";

		//now do 3' end of sequence submitted
		
      	energy = 0;
		for (j=ct->numofbases;j>(0)&&j>ct->numofbases-2;j--) {
          	energy = energy - data->stack[complement(j,ct)][ct->numseq[j]][complement(j-1,ct)][ct->numseq[j-1]];
		}
		//check for AU/GU ends
		if (ct->numseq[ct->numofbases]==4||ct->numseq[ct->numofbases]==1) {
			energy = energy - data->auend;
		}
		if (ct->numofbases>2) {
			if (ct->numseq[ct->numofbases-2]==4||ct->numseq[ct->numofbases-2]==1) {
				energy = energy + data->auend;
			}
		}
		
		//gcvt((float (energy))/10,6,temp);
		if (conversionfactor==10) sprintf(temp,"%.1f",(float (energy))/conversionfactor);
		else sprintf(temp,"%.2f",(float (energy))/conversionfactor); //assume conversionfactor==100


		out << temp<<"\n";


		delete ct;

		in >> temp;

		
	}

	in.close();
	out.close();

	return 0;//no errors
}
