/*
 * A header file for a program that analyzes a strand of nucleic acids using
 * stochastic probability sampling.
 * This strand of nucleic acids can be composed of either DNA or RNA.
 *
 * (c) 2009  Mathews Lab, University of Rochester Medical Center
 * Written by Jessica Reuter
 */

#ifndef STOCHASTIC_H
#define STOCHASTIC_H

#include "../RNA_class/RNA.h"
#include "../src/ErrorChecker.h"
#include "../src/ParseCommandLine.h"

class stochastic {
 public:
  // Public constructor and methods

  /*
   * Name:        Constructor
   * Description: Initalizes all private variables
   */
  stochastic();

  /*
   * Name:        parse
   * Description: Parses command line arguments to determine what options are
   *              required for a particular stochastic sampling calculation.
   * Arguments:
   *     1.   The number of command line arguments
   *     2.   The command line arguments themselves
   * Returns:
   *     true if parsing completed without errors, false if not
   */
  bool parse( int argc, char** argv );

  /*
   * Name:        run
   * Description: Run stochastic sampling calculations
   */
  void run();

 private:
  // Description of the calculation type
  string calcType;

  // Boolean, true if RNA, false if DNA
  bool isRNA;

  // Input and output file names
  string inFile;           // The input partition function save file
  string ctFile;           // The output ct file

  // Suboptimal structure parameters
  int ensemble;            // The ensemble size
  int seed;                // The random seed

  // Boolean that tells if the input file is a sequence file (true) or a pfs
  // file (false)
  bool isSequence;
};

#endif /* STOCHASTIC_H */
