/////////////////////////////////////////////////////////////////
// Parameters.h
/////////////////////////////////////////////////////////////////

#ifndef PARAMETERS_H
#define PARAMETERS_H

#include <fstream>
#include <algorithm>
#include <iostream>
#include <iomanip>
#include <sstream>
#include <vector>
#include <string>
#include <map>
#include <cstdlib>
#include "Score.h"
#include "Utilities.h"

using namespace std;

/////////////////////////////////////////////////////////////////
// Pre-defined parameter sets
/////////////////////////////////////////////////////////////////

// #define STANDARD_PARAMETER_SET
// #define MFOLD_PARAMETER_SET
#define ALL_PARAMETER_SET

/////////////////////////////////////////////////////////////////
// Standard parameter set
/////////////////////////////////////////////////////////////////

#ifdef STANDARD_PARAMETER_SET

#endif

/////////////////////////////////////////////////////////////////
// MFOLD-like parameter set
/////////////////////////////////////////////////////////////////

#ifdef MFOLD_PARAMETER_SET

#endif

/////////////////////////////////////////////////////////////////
// All parameters
/////////////////////////////////////////////////////////////////

#ifdef ALL_PARAMETER_SET

#define PARAMS_OUTER_LENGTH
#define PARAMS_HELIX_BASE_PAIR
#define PARAMS_HELIX_CLOSING

#define PARAMS_HELIX_LENGTH

#define PARAMS_HELIX_STACKING
#define PARAMS_TERMINAL_MISMATCH
#define PARAMS_HAIRPIN_LENGTH
#define PARAMS_BULGE_LENGTH
#define PARAMS_INTERNAL_FULL
#define PARAMS_INTERNAL_LENGTH
#define PARAMS_INTERNAL_ASYMMETRY
#define PARAMS_MULTI_LENGTH
#define PARAMS_SINGLE_BASE_PAIR_STACKING
#endif

const int C = 30;                  // max internal loop length

#ifdef PARAMS_HELIX_LENGTH
const int D = 5;                   // max length for explicit helix distribution
#else
const int D = 1;
#endif

const string alphabet = "ACGU";    // allowed symbols -- all other letters ignored
const int M = alphabet.length();   // number of alphabet symbols

/////////////////////////////////////////////////////////////////
// struct ParameterGroup
/////////////////////////////////////////////////////////////////

struct ParameterGroup {
  string name;
  int begin, end;
  
  ParameterGroup();
  ParameterGroup (const string &name, int begin, int end);
  ParameterGroup (const ParameterGroup &rhs);  
};

/////////////////////////////////////////////////////////////////
// class Parameters
/////////////////////////////////////////////////////////////////

class Parameters {
  vector<double> values;
  vector<string> names;

  vector<ParameterGroup> param_groups;
  string group_name;
  int group_begin;

  map<string, int> parameter_name_to_index;
  VPII regularization_list;
  
  void BeginGroup (const string &name);
  void EndGroup ();

  void NewParameter (const string &s);
  void AddParameterAlias (const string &alias, const string &name);

 public:

  Parameters();
  void InitializeRandomly();
  void InitializeDefaults();
  void Clear();

  int GetNumValues() const;
  const vector<double> GetValues() const;
  Parameters &LoadValues (const vector<double> &values);
  void ReadFromFile (const string &filename);
  void WriteToFile (const string &filename);

  Score GetParam (const string &s) const;
  void AccumulateCounts (const string &s, Score value);

  double ComputeRegularizationPenalty () const;
  const vector<double> ComputeRegularizationGradient () const;

  void RegularizeToZero (const string &s);
  void RegularizeTogether (const string &s, const string &t);

  const vector<ParameterGroup> &GetParameterGroups() const;
};

#endif
