/////////////////////////////////////////////////////////////////
// Table.h
/////////////////////////////////////////////////////////////////

#ifndef TABLE_H
#define TABLE_H

#include <cstdio>
#include <cstring>
#include <iomanip>
#include <iostream>
#include <fstream>
#include "Score.h"
#include "SStruct.h"
#include "Utilities.h"

/////////////////////////////////////////////////////////////////
// class Table
/////////////////////////////////////////////////////////////////

class Table {
  
  int layers;
  int size;
  
  VVVF data;
  
  void PrintVal (ostream &outfile, const Score &value) const;

 public:
  
  Table (int layers, int size);
  Table (const Table &rhs);

  void Fill (const Score &value);
  void PrintLayer (ostream &outfile, int layer) const;
  void Print (ostream &outfile) const;
  void PrintSparse (ostream &outfile, const string &seq) const;

  /////////////////////////////////////////////////////////////////
  // Table::operator()
  //
  // Access matrix element.
  /////////////////////////////////////////////////////////////////
  
  Score &operator() (int layer, int left, int right){
    Assert (0 <= layer && layer < layers, "Requested layer out-of-bounds.");
    Assert (0 <= left && left <= right && right < size, "Requested entry undefined.");
    return data[layer][left][right];
  }

  /////////////////////////////////////////////////////////////////
  // Table::operator()
  //
  // Access matrix element.(const version)
  /////////////////////////////////////////////////////////////////

  Score operator() (int layer, int left, int right) const {
    Assert (0 <= layer && layer < layers, "Requested layer out-of-bounds.");
    Assert (0 <= left && left <= right && right < size, "Requested entry undefined.");
    return data[layer][left][right];
  }

  int GetNumLayers() const { return layers; }
  int GetSize() const { return size; }
};

#endif
