/////////////////////////////////////////////////////////////////
// InnerOptimizationWrapper.hpp
//
// Inner optimization algorithm.
/////////////////////////////////////////////////////////////////

#ifndef INNEROPTIMIZATIONWRAPPER_HPP
#define INNEROPTIMIZATIONWRAPPER_HPP

#include "Computation.hpp"
#include "OptimizationWrapper.hpp"
#include "InnerOptimizationWrapper.hpp"
#include "LBFGS.hpp"
#include <limits>
class OptimizationWrapper;

/////////////////////////////////////////////////////////////////
// class InnerOptimizationWrapper
/////////////////////////////////////////////////////////////////

class InnerOptimizationWrapper : public LBFGS<double>
{
    OptimizationWrapper *optimizer;
    const std::vector<int> units;
    const std::vector<double> C;
    std::vector<double> best_x;
    double best_f;  

public:
    InnerOptimizationWrapper(OptimizationWrapper *optimizer,
                             const std::vector<int> &units,
                             const std::vector<double> &C);

    double ComputeFunction(const std::vector<double> &x);
    void ComputeGradient(std::vector<double> &g, const std::vector<double> &x);
    void Report(int iteration, const std::vector<double> &x, double f, double step_size);
    void Report(const std::string &s);
};

#endif
