#!/usr/bin/perl

use strict;

sub WriteDefaultValues 
{
    my ($filename, $function_name) = @_;
    open (INFILE, "<$filename");

    my @names = ();
    my @values = ();
    
    while (my $line = <INFILE>)
    {
	$line =~ /(\S+)\s+(\S+)/;
	push (@names, $1);
	push (@values, $2);
    }
    my $length = @names;

    close INFILE;

    print OUTFILE "/////////////////////////////////////////////////////////////////\n";
    print OUTFILE "// Parameters::$function_name()\n";
    print OUTFILE "//\n";
    print OUTFILE "// Retrieve default parameter values.\n";
    print OUTFILE "/////////////////////////////////////////////////////////////////\n";
    print OUTFILE "\n";
    print OUTFILE "std::vector<double> Parameters::$function_name()\n";
    print OUTFILE "{\n";
    print OUTFILE "    std::map<std::string, double> params;\n";
    print OUTFILE "\n";
    print OUTFILE "    char *default_names[$length] = {\n";
    for (my $i = 0; $i < @names; ++$i)
    {
	print OUTFILE "        \"$names[$i]\"".($i < $length-1 ? "," : "")."\n";
    }
    print OUTFILE "    };\n";
    print OUTFILE "\n";
    print OUTFILE "    double default_values[$length] =\n";
    print OUTFILE "    {\n";
    for (my $i = 0; $i < @values; ++$i)
    {
	print OUTFILE "        $values[$i]".($i < $length-1 ? "," : "")."\n";
    }
    print OUTFILE "    };\n";
    print OUTFILE "\n";
    print OUTFILE "    for (size_t i = 0; i < $length; i++)\n";
    print OUTFILE "        params[default_names[i]] = default_values[i];\n";
    print OUTFILE "\n";
    print OUTFILE "    std::vector<double> values(names.size());\n";
    print OUTFILE "    for (size_t i = 0; i < names.size(); i++)\n";
    print OUTFILE "    {\n";
    print OUTFILE "        std::map<std::string, double>::iterator iter = params.find(names[i]);\n";
    print OUTFILE "        if (iter == params.end())\n";
    print OUTFILE "            Error(\"Default parameter file missing parameter: %s\", names[i].c_str());\n";
    print OUTFILE "        values[i] = iter->second;\n";
    print OUTFILE "        params.erase(iter);\n";
    print OUTFILE "    }\n";
    print OUTFILE "\n";
    print OUTFILE "    for (std::map<std::string, double>::iterator iter = params.begin(); iter != params.end(); ++iter)\n";
    print OUTFILE "        Warning(\"Default parameter file contains extra parameter: %s\", iter->first.c_str());\n";
    print OUTFILE "\n";
    print OUTFILE "    return values;\n";
    print OUTFILE "}\n";    
}

sub main 
{
    if (@ARGV != 2)
    {
	print STDERR "Usage: perl MakeDefaults.pl PARAMFILE PARAMFILE_COMPLEMENTARY\n";
	exit(1);
    }

    open(OUTFILE, ">Defaults.hpp");
    WriteDefaultValues($ARGV[0], "GetDefaultValues");
    print OUTFILE "\n";
    WriteDefaultValues($ARGV[1], "GetDefaultComplementaryValues");
    close OUTFILE;
}

main();
