/////////////////////////////////////////////////////////////////
// OptimizationWrapper.hpp
/////////////////////////////////////////////////////////////////

#ifndef OPTIMIZATIONWRAPPER_HPP
#define OPTIMIZATIONWRAPPER_HPP

#include "Config.hpp"
#include "Utilities.hpp"
#include "Computation.hpp"
#include "Parameters.hpp"
#include "CGOptimizationWrapper.hpp"
#include "InnerOptimizationWrapper.hpp"
#include "OuterOptimizationWrapper.hpp"

/////////////////////////////////////////////////////////////////
// class OptimizationWrapper
//
// Wrapper class for performing optimization.
/////////////////////////////////////////////////////////////////

class OptimizationWrapper
{
    friend class CGOptimizationWrapper;
    friend class InnerOptimizationWrapper;
    friend class OuterOptimizationWrapper;
    
    Computation &computation;
    Parameters &params;
    std::vector<double> base_values;
    int indent;
    std::ofstream logfile;
    
    void Indent();
    void Unindent();
    void PrintMessage(const std::string &s);
    
    std::vector<double> TransformHyperparameters(const std::vector<double> &C);
    std::vector<double> TransformHyperparametersDerivative(const std::vector<double> &C);
    
public:
    
    OptimizationWrapper(Computation &computation,
                        Parameters &params,
                        const std::vector<double> &base_values);
    ~OptimizationWrapper();
    
    void Train(const std::vector<int> &units,
               std::vector<double> &values,
               const std::vector<double> &C,
               bool toggle_viterbi);
    
    void LearnHyperparameters(std::vector<int> units,
                              std::vector<double> &values,
                              double holdout_ratio,
                              bool toggle_viterbi);
};  

#endif
