/////////////////////////////////////////////////////////////////
// OuterOptimizationWrapper.hpp
//
// Outer optimization algorithm.
/////////////////////////////////////////////////////////////////

#ifndef OUTEROPTIMIZATIONWRAPPER_HPP
#define OUTEROPTIMIZATIONWRAPPER_HPP

#include "Computation.hpp"
#include "OptimizationWrapper.hpp"
#include "CGOptimizationWrapper.hpp"
#include "InnerOptimizationWrapper.hpp"
#include "OuterOptimizationWrapper.hpp"
#include "LBFGS.hpp"

class OptimizationWrapper;

/////////////////////////////////////////////////////////////////
// class OuterOptimizationWrapper
/////////////////////////////////////////////////////////////////

class OuterOptimizationWrapper : public LBFGS<double>
{
    OptimizationWrapper *optimizer;
    const std::vector<double> initial_values;
    const std::vector<int> training;
    const std::vector<int> holdout;
    
public:
    OuterOptimizationWrapper(OptimizationWrapper *optimizer,
                             const std::vector<double> &initial_values,
                             const std::vector<int> &training,
                             const std::vector<int> &holdout);

    double ComputeFunction(const std::vector<double> &x);
    void ComputeGradient(std::vector<double> &g, const std::vector<double> &x);
    void Report(int iteration, const std::vector<double> &x, double f, double step_size);
    void Report(const std::string &s);
};

#endif
