/////////////////////////////////////////////////////////////////
// SStruct.hpp
/////////////////////////////////////////////////////////////////

#ifndef SSTRUCT_HPP
#define SSTRUCT_HPP

#include <fstream>
#include <vector>
#include <string>
#include <sstream>
#include "Utilities.hpp"

const int UNKNOWN = -1;
const int UNPAIRED = 0;

/////////////////////////////////////////////////////////////////
// enum FileFormat
/////////////////////////////////////////////////////////////////

enum FileFormat
{ 
    FileFormat_FASTA,
    FileFormat_BPSEQ,
    FileFormat_RAW
};                  

/////////////////////////////////////////////////////////////////
// class SStruct
/////////////////////////////////////////////////////////////////

class SStruct
{
    std::string sequence;
    std::vector<int> mapping;
    std::string name;
    
    void AssertValidSequence(const std::string &sequence) const;
    void AssertValidMapping(const std::vector<int> &mapping) const;
    
public:
    
    SStruct(const std::string &filename);
    SStruct(const std::string &sequence, const std::vector<int> &mapping, const std::string &name);
    SStruct(const SStruct &rhs);
    const SStruct& operator=(const SStruct &rhs);
    ~SStruct();
    
    void LoadFASTA(const std::string &filename);
    void LoadRAW(const std::string &filename);
    void LoadBPSEQ(const std::string &filename);
    
    void WriteBPSEQ(std::ostream &outfile, bool toggle_complementary_only) const;
    void WriteParens(std::ostream &outfile, bool toggle_complementary_only) const;
    
    static bool IsComplementary(char c, char d);
    std::vector<int> GetComplementaryMapping() const;
    
    /////////////////////////////////////////////////////////////////
    // Getters
    /////////////////////////////////////////////////////////////////
    
    const std::string &GetSequence() const { return sequence; }
    const std::vector<int> &GetMapping() const { return mapping; }
    const std::string &GetName() const { return name; }
    int GetLength() const { return mapping.size()-1; }
};

#endif
